###########################################################################################
# Simulation 1, Known testing error rates, 5000 individuals, and the following three models
#
# Model 1: logit(pi) = beta0 + beta1*X1,              beta0=-3 and beta1=2 
# Model 2: logit(pi) = beta0 + beta1*X1 + beta2*X1^2, beta0=-3, beta1=2, and beta2=-0.5
# Model 3: logit(pi) = beta0 + beta1*X1 + beta2*X2,   beta0=-3, beta1=2, and beta2=-1
#
# where X1~N(0,1) and X2~Bernoulli(0.5). Pools of size c=5 are used, under MPT, DT, and AT.


#############################################################################################
# Read in needed functions Note: Set R directory to the file that conatins the necessary dlls

source("MHS_GTR_UP.txt")
source("MHS_IND_UP.txt")
source("Testing functions.txt")


###########################################################
# Specify the number of Gibbs iterates and prior parameters
GI<-5000  # Number of Gibbs iterates

###################
# Simulation settings
N<-5000                 # Number of Individuals
sims<-500               # Number of simulations
Se.t<-.95               # Sensitivity
Sp.t<-.98               # Specificity
beta.t1<-c(-3,2)        # Model 1 parameters
beta.t2<-c(-3,2,-0.5)   # Model 2 parameters
beta.t3<-c(-3,2,-1)     # Model 3 parameters
cj<-5                   # Pool size, note array testing uses cjxcj arrays

save<-seq(1000,GI,5)
ns<-length(save)
np1<-length(beta.t1)
np2<-length(beta.t2)
np3<-length(beta.t3)

########################################
# Saving devices for model 1
M1.indv<-array(-99,c(ns,np1,sims)) 
M1.pool<-array(-99,c(ns,np1,sims))
M1.dorf<-array(-99,c(ns,np1,sims))
M1.array<-array(-99,c(ns,np1,sims))
M1.cost.dorf<-array(-99,c(1,sims))
M1.cost.array<-array(-99,c(1,sims))

########################################
# Saving devices for model 2
M2.indv<-array(-99,c(ns,np2,sims)) 
M2.pool<-array(-99,c(ns,np2,sims))
M2.dorf<-array(-99,c(ns,np2,sims))
M2.array<-array(-99,c(ns,np2,sims))
M2.cost.dorf<-array(-99,c(1,sims))
M2.cost.array<-array(-99,c(1,sims))

########################################
# Saving devices for model 1
M3.indv<-array(-99,c(ns,np3,sims)) 
M3.pool<-array(-99,c(ns,np3,sims))
M3.dorf<-array(-99,c(ns,np3,sims))
M3.array<-array(-99,c(ns,np3,sims))
M3.cost.dorf<-array(-99,c(1,sims))
M3.cost.array<-array(-99,c(1,sims))

set.seed(12345)

for(mod in 1:3){
for(s in 1:sims){

#######################################
# Generate true statuses
if(mod==1){
X<-cbind(rep(1,N),rnorm(N,0,1))
p<-exp(X%*%beta.t1)/(1+exp(X%*%beta.t1))
Y.true<-rbinom(N,1,p)
}

if(mod==2){
X<-cbind(rep(1,N),rnorm(N,0,1))
X<-cbind(X,X[,2]^2)
p<-exp(X%*%beta.t2)/(1+exp(X%*%beta.t2))
Y.true<-rbinom(N,1,p)
}

if(mod==3){
X<-cbind(rep(1,N),rnorm(N,0,1),rbinom(N,1,0.5))
p<-exp(X%*%beta.t3)/(1+exp(X%*%beta.t3))
Y.true<-rbinom(N,1,p)
}

#################################################
# Simulates individual testing
test.res<-Ind.decode(Y.true,Se.t,Sp.t)
Y.ind<-test.res

#################################################################################################################################################
# Individual testing responses only with testing error setting 1 known
if(mod==1){
res<-Bayes.reg.ind(Y=Y.ind, X=X, GI=GI, b0=beta.t1, Se=Se.t, Sp=Sp.t, visual=FALSE)
M1.indv[,,s]<-res$beta[save,]
}
if(mod==2){
res<-Bayes.reg.ind(Y=Y.ind, X=X, GI=GI, b0=beta.t2, Se=Se.t, Sp=Sp.t, visual=FALSE)
M2.indv[,,s]<-res$beta[save,]
}
if(mod==3){
res<-Bayes.reg.ind(Y=Y.ind, X=X, GI=GI, b0=beta.t3, Se=Se.t, Sp=Sp.t, visual=FALSE)
M3.indv[,,s]<-res$beta[save,]
}

#################################################
# Simulate master pool testing 
test.res<-Pool.test(Y.true,Se.t,Sp.t,cj=cj)
Z.pool<-test.res$Z
Y.pool<-test.res$Y

#################################################################################################################################################
# Initial pool response only with testing errors known
if(mod==1){
res<-Bayes.reg.GT(Z=Z.pool, Y=Y.pool, X=X, GI=GI, b0=beta.t1, na=1, Se=Se.t, Sp=Sp.t, visual=FALSE)
M1.pool[,,s]<-res$beta[save,]
}
if(mod==2){
res<-Bayes.reg.GT(Z=Z.pool, Y=Y.pool, X=X, GI=GI, b0=beta.t2, na=1, Se=Se.t, Sp=Sp.t, visual=FALSE)
M2.pool[,,s]<-res$beta[save,]
}
if(mod==3){
res<-Bayes.reg.GT(Z=Z.pool, Y=Y.pool, X=X, GI=GI, b0=beta.t3, na=1, Se=Se.t, Sp=Sp.t, visual=FALSE)
M3.pool[,,s]<-res$beta[save,]
}

#################################################################
# Simulates Dorfman testing
test.res<-Dorfman.decode.same.error(Y.true,Se.t,Sp.t,cj=cj)
Z.dorf<-test.res$Z
Y.dorf<-test.res$Y

###################################################################################################################################################################
# Data from Dorfman decoding with testing error rates known
if(mod==1){
res<-Bayes.reg.GT(Z=Z.dorf, Y=Y.dorf, X=X, GI=GI, b0=beta.t1, na=1, Se=Se.t, Sp=Sp.t, visual=FALSE)
M1.dorf[,,s]<-res$beta[save,]
M1.cost.dorf[,s]<-dim(Z.dorf)[1]
}
if(mod==2){
res<-Bayes.reg.GT(Z=Z.dorf, Y=Y.dorf, X=X, GI=GI, b0=beta.t2, na=1, Se=Se.t, Sp=Sp.t, visual=FALSE)
M2.dorf[,,s]<-res$beta[save,]
M2.cost.dorf[,s]<-dim(Z.dorf)[1]
}
if(mod==3){
res<-Bayes.reg.GT(Z=Z.dorf, Y=Y.dorf, X=X, GI=GI, b0=beta.t3, na=1, Se=Se.t, Sp=Sp.t, visual=FALSE)
M3.dorf[,,s]<-res$beta[save,]
M3.cost.dorf[,s]<-dim(Z.dorf)[1]
}

#########################################################################################################################################################
# Simulates array testing  
test.res<-Array.decode.same.error(Y.true, Se.t, Sp.t, cj=cj)
Z.array<-test.res$Z
Y.array<-test.res$Y

###################################################################################################################################################################
# Data from Array decoding with testing error rates known
if(mod==1){
res<-Bayes.reg.GT(Z=Z.array, Y=Y.array, X=X, GI=GI, b0=beta.t1, na=1, Se=Se.t, Sp=Sp.t, visual=FALSE)
M1.array[,,s]<-res$beta[save,]
M1.cost.array[,s]<-dim(Z.array)[1]
}
if(mod==2){
res<-Bayes.reg.GT(Z=Z.array, Y=Y.array, X=X, GI=GI, b0=beta.t2, na=1, Se=Se.t, Sp=Sp.t, visual=FALSE)
M2.array[,,s]<-res$beta[save,]
M2.cost.array[,s]<-dim(Z.array)[1]
}
if(mod==3){
res<-Bayes.reg.GT(Z=Z.array, Y=Y.array, X=X, GI=GI, b0=beta.t3, na=1, Se=Se.t, Sp=Sp.t, visual=FALSE)
M3.array[,,s]<-res$beta[save,]
M3.cost.array[,s]<-dim(Z.array)[1]
}

}} # End of simulation




########################################################
# Number of tests expended
# Individual testing  = 5000
# master pool testing = 1000

mean(M1.cost.dorf)
mean(M2.cost.dorf)
mean(M3.cost.dorf)

mean(M1.cost.array)
mean(M2.cost.array)
mean(M3.cost.array)

########################################################
# Summarize results for model 1

M1.point.indv<-t(apply(M1.indv,c(2,3),mean))
M1.point.pool<-t(apply(M1.pool,c(2,3),mean))
M1.point.dorf<-t(apply(M1.dorf,c(2,3),mean))
M1.point.array<-t(apply(M1.array,c(2,3),mean))

M1.sd.indv<-t(apply(M1.indv,c(2,3),sd))
M1.sd.pool<-t(apply(M1.pool,c(2,3),sd))
M1.sd.dorf<-t(apply(M1.dorf,c(2,3),sd))
M1.sd.array<-t(apply(M1.array,c(2,3),sd))


round(apply(M1.point.indv,2,mean)-beta.t1,2)
round(apply(M1.point.indv,2,sd),2)
round(apply(M1.sd.indv,2,mean),2)
cov.func(M1.indv,sims,2,0.05,beta.t1)

round(apply(M1.point.pool,2,mean)-beta.t1,2)
round(apply(M1.point.pool,2,sd),2)
round(apply(M1.sd.pool,2,mean),2)
cov.func(M1.pool,sims,2,0.05,beta.t1)

round(apply(M1.point.dorf,2,mean)-beta.t1,2)
round(apply(M1.point.dorf,2,sd),2)
round(apply(M1.sd.dorf,2,mean),2)
cov.func(M1.dorf,sims,2,0.05,beta.t1)

round(apply(M1.point.array,2,mean)-beta.t1,2)
round(apply(M1.point.array,2,sd),2)
round(apply(M1.sd.array,2,mean),2)
cov.func(M1.array,sims,2,0.05,beta.t1)


########################################################
# Summarize results for model 2

M2.point.indv<-t(apply(M2.indv,c(2,3),mean))
M2.point.pool<-t(apply(M2.pool,c(2,3),mean))
M2.point.dorf<-t(apply(M2.dorf,c(2,3),mean))
M2.point.array<-t(apply(M2.array,c(2,3),mean))

M2.sd.indv<-t(apply(M2.indv,c(2,3),sd))
M2.sd.pool<-t(apply(M2.pool,c(2,3),sd))
M2.sd.dorf<-t(apply(M2.dorf,c(2,3),sd))
M2.sd.array<-t(apply(M2.array,c(2,3),sd))


round(apply(M2.point.indv,2,mean)-beta.t2,2)
round(apply(M2.point.indv,2,sd),2)
round(apply(M2.sd.indv,2,mean),2)
cov.func(M2.indv,sims,3,0.05,beta.t2)


round(apply(M2.point.pool,2,mean)-beta.t2,2)
round(apply(M2.point.pool,2,sd),2)
round(apply(M2.sd.pool,2,mean),2)
cov.func(M2.pool,sims,3,0.05,beta.t2)

round(apply(M2.point.dorf,2,mean)-beta.t2,2)
round(apply(M2.point.dorf,2,sd),2)
round(apply(M2.sd.dorf,2,mean),2)
cov.func(M2.dorf,sims,3,0.05,beta.t2)

round(apply(M2.point.array,2,mean)-beta.t2,2)
round(apply(M2.point.array,2,sd),2)
round(apply(M2.sd.array,2,mean),2)
cov.func(M2.array,sims,3,0.05,beta.t2)

########################################################
# Summarize results for model 3

M3.point.indv<-t(apply(M3.indv,c(2,3),mean))
M3.point.pool<-t(apply(M3.pool,c(2,3),mean))
M3.point.dorf<-t(apply(M3.dorf,c(2,3),mean))
M3.point.array<-t(apply(M3.array,c(2,3),mean))

M3.sd.indv<-t(apply(M3.indv,c(2,3),sd))
M3.sd.pool<-t(apply(M3.pool,c(2,3),sd))
M3.sd.dorf<-t(apply(M3.dorf,c(2,3),sd))
M3.sd.array<-t(apply(M3.array,c(2,3),sd))


round(apply(M3.point.indv,2,mean)-beta.t3,2)
round(apply(M3.point.indv,2,sd),2)
round(apply(M3.sd.indv,2,mean),2)
cov.func(M3.indv,sims,3,0.05,beta.t3)

round(apply(M3.point.pool,2,mean)-beta.t3,2)
round(apply(M3.point.pool,2,sd),2)
round(apply(M3.sd.pool,2,mean),2)
cov.func(M3.pool,sims,3,0.05,beta.t3)

round(apply(M3.point.dorf,2,mean)-beta.t3,2)
round(apply(M3.point.dorf,2,sd),2)
round(apply(M3.sd.dorf,2,mean),2)
cov.func(M3.dorf,sims,3,0.05,beta.t3)

round(apply(M3.point.array,2,mean)-beta.t3,2)
round(apply(M3.point.array,2,sd),2)
round(apply(M3.sd.array,2,mean),2)
cov.func(M3.array,sims,3,0.05,beta.t3)















